@@ -2,7 +2,7 @@ package Proc::PersistentControl; # -*-perl-*-
 #
 # Author: Michael Staats 2014
 #
-# $Id: PersistentControl.pm 711 2015-03-11 14:29:23Z michael $
+# $Id: PersistentControl.pm 713 2015-03-13 12:05:17Z michael $
 #
 
 =head1 NAME
@@ -228,7 +228,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @ISA = qw(Exporter);
 
-$VERSION = '1.0';
+$VERSION = '1.2';
 
 use strict;
 use File::Path qw(mkpath rmtree); # legacy interface, works with older perls
@@ -452,13 +452,18 @@ sub _unix_spawn {
 	    my $waited = 0;
 	    debug("waiting max $psynctime s for 'start' file");
 	    for (my $nwait = 1; $nwait < 10 * $psynctime; $nwait++) {
-		last if (-e $pdir . "/start");
+		last if (-e $pdir . "/pstart");
 		$waited++;
 		select(undef, undef, undef, 0.1);
 	    }
+	    unlink($pdir . "/pstart");
 	    debug("waited " . (0.1 * $waited) . " s for 'start' file");
-	    exec(@cmd) or
+	    open(STA, '>', $pdir . "/start"); print STA time(), "\n"; close(STA);
+	    if (not exec(@cmd)) {
+		open(FA, '>', $pdir . "/fail"); print FA "$!\n"; close(FA);
+		unlink($pdir . "/start");
 		croak "Could not exec(", join(', ', @cmd), ")\n";
+	    }
 	}
 	croak "Cannot fork(): $!\n" if (not defined $_unix_grandch_pid);
 
@@ -467,7 +472,7 @@ sub _unix_spawn {
 	$SIG{QUIT} = \&_unix_term_sighandler;
 
 	open(PID, '>', $pdir . "/pid=$_unix_grandch_pid"); close(PID);	
-	open(STA, '>', $pdir . "/start"); print STA time(), "\n"; close(STA);
+	open(STA, '>', $pdir . "/pstart"); print STA time(), "\n"; close(STA);
 	debug("child says: \$_unix_grandch_pid = $_unix_grandch_pid " .
 	      "for $cmd[0] ...");
 	
@@ -770,12 +775,18 @@ sub StartProc {
     my $psynctime = $opt->{psynctime} || 10;
     my $waited = 0;
     for (my $nwait = 1; $nwait < 10 * $psynctime; $nwait++) {
-	last if (-e $psd . "/start");
+	last if (-e $psd . "/start" or -e $psd . "/fail");
 	$waited++;
 	select(undef, undef, undef, 0.1);
     }
-    debug("waited " . (0.1 * $waited) . " s for 'start' file");
-    if (-e $psd . "/start") {
+    debug("waited " . (0.1 * $waited) . " s for 'start/fail' file");
+    if (-e $psd . "/fail") {
+	open(FA, $psd . "/fail") or die "Cannot read $psd/fail: $!\n";
+	my $st = <FA>;
+	close(FA);
+	carp "StartProc could not start child process $cmd[0]: $st";
+	return undef;
+    } elsif (-e $psd . "/start") {
 	bless $Proc, 'Proc::PersistentControl::Proc';
 	debug("returning Proc object for $UID, pid file " .
 	      (glob($psd . '/pid=*'))[0]);
